import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:healthrian_ble_interface/healthrian_ble_interface.dart';
import 'package:healthrian_common_support/healthrian_common_support.dart';

abstract class ManagerInterface {
  // ModuleInterface get module => modules[index];
  // int index = 0;
  // final modules = <ModuleInterface>[];
  late ModuleInterface module;

  @Deprecated('Use FlutterBluePlus directly without instantiation')
  FlutterBluePlusWrapper raw = FlutterBluePlusWrapper();

  ///////////////////

  ModuleBle get ble => module.ble;

  ModuleEcg? get ecg => module.ecg;

  ModuleOxi? get oxi => module.oxi;

  ///////////

  set scanFilter(bool Function(BluetoothDevice device) scanFilter) {
    module.ble = module.ble.copyWith(scanFilter: scanFilter);
    module.ecg = module.ecg?.copyWith(ble: module.ble);
    module.oxi = module.oxi?.copyWith(ble: module.ble);
  }

  bool Function(BluetoothDevice device) get scanFilter => module.ble.scanFilter;

  StreamInterface get streams => module.ble.streams;

  BluetoothDevice? get device => module.ble.device;

  set device(BluetoothDevice? value) {
    module.ble.device = value;
  }

  // List<BluetoothDevice> get devices => module.ble.devices;

  Future<bool> connectDevice([
    String preferredDeviceId = '',
    bool strictConnection = false,
  ]) async {
    return await module.ble.connectDevice(
      preferredDeviceId,
      strictConnection,
    );
  }

  Future<void> disconnectDevice() async {
    await module.ble.disconnectDevice();
  }

  Future<void> registerCharacteristics() async {
    await module.ble.registerCharacteristics();
  }

  Future<void> initializeStreams() async {
    module.ecg?.initStream();
    module.oxi?.initStream();
  }

  Future<void> startMeasurement() async {
    await module.ecg?.start();
    await module.oxi?.start();
  }

  Future<void> stopMeasurement() async {
    await module.ecg?.stop();
    await module.oxi?.stop();
  }

  Future<void> pauseMeasurement() async {
    await module.ecg?.pause();
    await module.oxi?.pause();
  }

  Future<Channels?> saveMeasurement({ModuleType type = ModuleType.ecg}) async {
    if (type == ModuleType.ecg) {
      final lastChannel = module.ecg?.lastChannel;
      final dataFilter = module.ecg?.dataFilter;
      if (lastChannel == null) return null;
      return Channels.fromList(lastChannel.list.toList(), dataFilter);
    }

    if (type == ModuleType.oxi) {
      final lastChannel = module.oxi?.lastChannel;
      final dataFilter = module.oxi?.dataFilter;
      if (lastChannel == null) return null;
      return Channels.fromList(lastChannel.list.toList(), dataFilter);
    }

    return null;
  }
}
